from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class ShareChatBot(ChatBotAPI, metaclass=Builder):
    # 分享名片消息 https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN#756b882f
    _templates = {
        "chat_id": {
            Required("msg_type"): "share_chat",
            Required("content"): {
                Required("chat_id"): str,
            },
            "receive_id": str
        },
        "user_id": {
            Required("msg_type"): "share_user",
            Required("content"): {
                Required("user_id"): str,
            },
            "receive_id": str
        }
    }



from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class ImageChatBot(ChatBotAPI, metaclass=Builder):
    # 图片消息 https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN#132a114c
    _templates = {
        "basic": {
            Required("msg_type"): "image",
            Required("content"): {
                Required("image_key"): str
            },
            "receive_id": str
        }
    }



from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class TextChatBot(ChatBotAPI, metaclass=Builder):
    # 文本消息 https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN#756b882f
    _templates = {
        "basic": {
            Required("msg_type"): "text",
            Required("content"): {
                Required("text"): str
            },
            "receive_id": str
        }
    }


import json

from chatbot.enums import OpenAPI
from chatbot.http_client import HttpClient


class ChatBotClient:
    def __init__(self, app_id=None, app_secret=None):
        if not app_id or not app_secret:
            raise ChatBotClientException("init client error, app_id, app_secret parameter is required")

        self.app_id = app_id
        self.app_secret = app_secret
        self.client = HttpClient()

    @classmethod
    def init_client(cls, app_id, app_secret):
        return cls(app_id, app_secret)

    def send(self, message, **kwargs):
        """
        发送消息
        :param message:
        :param kwargs:
        :return:
        """
        access_token = self._get_access_token()
        return self.client.post(
            url_params=kwargs,
            content_type="data",
            data=json.dumps(message),
            path=OpenAPI.SEND_MESSAGE.value,
            response_data_extractor=openapi_callback,
            headers={"Authorization": f"Bearer {access_token}", "Content-Type": "application/json; charset=utf-8"}
        )

    def _get_access_token(self):
        """
        获取 access token
        :return:
        """
        data = self.client.post(
            OpenAPI.ACCESS_TOKEN.value,
            response_data_extractor=openapi_callback,
            data={"app_id": self.app_id, "app_secret": self.app_secret},
            headers={"Content-Type": "application/json; charset=utf-8"}
        )
        print(data.get("tenant_access_token"))
        return data.get("tenant_access_token")

    def get_image_key(self, image, image_type="message"):
        """
        上传图片获取 image key
        :param image:
        :param image_type:
        :return:
        """
        access_token = self._get_access_token()
        data = self.client.post(
            content_type="data",
            path=OpenAPI.GET_IMAGE_KEY.value,
            response_data_extractor=openapi_callback,
            data={"image_type": image_type},
            files={'image': image},
            headers={"Authorization": f"Bearer {access_token}"}
        )
        return data["data"]["image_key"]

    def get_file_key(self, file, file_name, file_type):
        """
        上传文件获取 file key
        :param file: 二进制文件
        :param file_name: 带后缀的文件名称。例；测试视频.mp4
        :param file_type: 文件类型.
            opus：上传opus音频文件；其他格式的音频文件，请转为opus格式后上传，转换方式可参考：ffmpeg -i SourceFile.mp3 -acodec libopus -ac 1 -ar 16000 TargetFile.opus
            mp4：上传mp4视频文件
            pdf：上传pdf格式文件
            doc：上传doc格式文件
            xls：上传xls格式文件
            ppt：上传ppt格式文件
            stream：上传stream格式文件。以上类型之外，可以使用stream格式
        :return:
        """
        access_token = self._get_access_token()
        data = self.client.post(
            content_type="data",
            files={'file': file},
            path=OpenAPI.GET_FILE_KEY.value,
            response_data_extractor=openapi_callback,
            data={"file_name": file_name, "file_type": file_type},
            headers={"Authorization": f"Bearer {access_token}"}
        )
        return data["data"]["file_key"]

    def get_user_id_by_email(self, email: str = None):
        """
        根据邮箱获取用户 openid，userid
        :param email:
        :return:
        """
        return self._get_user_id(email=email)

    def get_user_id_by_mobile(self, mobile: str = None):
        """
        根据电话获取用户 openid，userid
        :param mobile:
        :return:
        """
        return self._get_user_id(mobile=mobile)

    def _get_user_id(self, email: str = None, mobile: str = None):
        """
        获取用户 openid，userid
        :param email:
        :param mobile:
        :return:
        """
        url_params = dict()
        access_token = self._get_access_token()

        if email:
            url_params.update({"emails": email})
        if mobile:
            url_params.update({"mobiles": mobile})
        data = self.client.get(
            url_params=url_params,
            path=OpenAPI.GET_USER_ID.value,
            response_data_extractor=openapi_callback,
            headers={"Authorization": f"Bearer {access_token}", "Content-Type": "application/json; charset=utf-8"}
        )
        if email:
            return list(data["data"]['email_users'].values())[0][0]
        return list(data["data"]['mobile_users'].values())[0][0]

    def get_group_robot_id(self):
        """
        获取机器人所在群列表
        :return:
        """
        access_token = self._get_access_token()
        data = self.client.get(
            OpenAPI.GET_GROUP_ROBOT_ID.value,
            response_data_extractor=openapi_callback,
            headers={"Authorization": f"Bearer {access_token}", "Content-Type": "application/json; charset=utf-8"}
        )
        return data["data"]["items"]


class ChatBotClientException(Exception):
    pass


def openapi_callback(response):
    response_data = response.json()
    if response.status_code != 200 or response_data.get("code") != 0:
        raise ChatBotClientException(
            f"openapi return error, code: {response_data.get('code')}, msg: {response_data.get('msg')}")
    return response_data



from enum import Enum


class OpenAPI(Enum):
    ACCESS_TOKEN = 'https://open.feishu.cn/open-apis/auth/v3/app_access_token/internal/'
    SEND_MESSAGE = 'https://open.feishu.cn/open-apis/im/v1/messages'
    REPLY_MESSAGE = 'https://open.feishu.cn/open-apis/im/v1/messages/{message_id}/reply'
    GET_USER_ID = 'https://open.feishu.cn/open-apis/user/v1/batch_get_id'
    GET_GROUP_ROBOT_ID = 'https://open.feishu.cn/open-apis/im/v1/chats'
    GET_IMAGE_KEY = 'https://open.feishu.cn/open-apis/im/v1/images'
    GET_FILE_KEY = 'https://open.feishu.cn/open-apis/im/v1/files'



from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class InteractiveChatBot(ChatBotAPI, metaclass=Builder):
    # 卡片消息 https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN#4996824a
    _templates = {
        "basic": {
            Required("msg_type"): "interactive",
            Required("content"): {
                Required("config"): dict,
                Required("elements"): list,
                Required("header"): dict
            },
            "receive_id": str
        }
    }



from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class PostChatBot(ChatBotAPI, metaclass=Builder):
    # 富文本消息 https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN#f62e72d5
    _templates = {
        "zh_cn": {
            Required("msg_type"): "post",
            Required("content"): {
                Required("post"): {
                    Required("zh_cn"): {
                        Required("title"): str,
                        Required("content"): list
                    }
                }
            },
            "receive_id": str
        },
        "en_us": {
            Required("msg_type"): "post",
            Required("content"): {
                Required("post"): {
                    Required("zh_cn"): {
                        Required("title"): str,
                        Required("content"): list
                    }
                }
            },
            "receive_id": str
        }
    }



import setuptools
from setuptools import setup
import io

with io.open('README.md', 'rt', encoding='utf8') as f:
    long_description = f.read()

setup(
    name='feishu_chatbot',
    version="0.0.4",
    install_requires=['voluptuous>=0.12.2', 'requests>=2.25.0'],
    url='https://github.com/rexyan/feishu_chatbot',
    license='MIT',
    author='RexYan',
    author_email='rex_yan@126.com',
    description='feishu chatbot api',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    platforms=['all'],
    include_package_data=True,
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ]

)


__version__ = '0.0.1'


from .client import *
from .api import *
from .api.audio import *
from .api.file import *
from .api.image import *
from .api.interactive import *
from .api.media import *
from .api.post import *
from .api.share_chat import *
from .api.sticker import *
from .api.text import *



"""
自定义 http 请求类：支持设置重试次数，重试退避规则，强制重试的一组 HTTP 状态代码，支持数据请求成功 callback

使用示例：
    def func(data):
        return data["resultcode"]

    http = HttpClient()
    print(http.get("http://xxx/api"))
    print(http.get("http://apis.juhe.cn/ip/ipNew"))
    print(http.get("http://apis.juhe.cn/ip/ipNew", response_data_extractor=func))
"""

import requests
from requests import Response
from requests.exceptions import (
    RetryError,
    HTTPError,
    ConnectionError,
    Timeout,
    ConnectTimeout,
    RequestException,
)
from urllib3 import Retry
from collections import defaultdict
from collections.abc import Callable
from requests.adapters import HTTPAdapter


class HttpClient:
    __slots__ = ()

    def __requests_retry_session(
        self,
        retries: int = 3,
        backoff_factor: float = 0.3,
        status_forcelist: tuple = (500, 502, 504),
        session: object = None,
    ) -> requests.Session:
        """
        获取 session 对象
        @param retries: 重试次数
        @param backoff_factor: 重试退避规则
        @param status_forcelist: 强制重试 HTTP 状态代码
        @param session:
        @return:
        """
        session = session or requests.Session()
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
        )
        adapter = HTTPAdapter(max_retries=retry)
        session.mount("http://", adapter)
        session.mount("https://", adapter)
        return session

    def get(
        self,
        path: str,
        headers: dict = None,
        url_params: dict = None,
        data: dict = None,
        response_data_extractor: Callable = lambda x: x,
        verify: bool = False,
    ) -> object or Response:
        """
        get 请求
        @param path: 请求路径
        @param headers: 请求头
        @param url_params: 请求数据
        @param data: 请求头
        @param response_data_extractor: 回调函数
        @param verify: 是否校验证书
        @return:
        """
        params = defaultdict()
        params["url"] = path
        params["verify"] = verify
        if headers:
            params["headers"] = headers
        if data:
            params["params"] = data
        if url_params:
            params["params"] = url_params
        try:
            response = self.__requests_retry_session().get(**params)
            response.raise_for_status()
            result = response_data_extractor(response)
        except (RetryError, HTTPError, ConnectionError, Timeout, ConnectTimeout) as e:
            raise RequestException(f"HttpClient Request Error: {e}, {e.response.text}")
        except Exception as e:
            raise RequestException(f"HttpClient Error: {e}")
        else:
            return result

    def post(
        self,
        path: str,
        data: dict or str = None,
        headers: dict = None,
        files=None,
        url_params: dict = None,
        response_data_extractor: Callable = lambda x: x,
        content_type: str = "json",
        verify: bool = False,
    ) -> object or Response:
        """
        post 请求
        @param path: 请求路径
        @param data: 请求数据
        @param url_params: 请求数据
        @param headers: 请求头
        @param files: 文件
        @param response_data_extractor: 回调函数
        @param content_type: 请求类型
        @param verify: 是否校验证书
        @return:
        """
        params = defaultdict()
        params["url"] = path
        params["verify"] = verify
        if headers:
            params["headers"] = headers
        if data:
            params[content_type] = data
        if url_params:
            params["params"] = url_params
        if files:
            params["files"] = files
        try:
            response = self.__requests_retry_session().post(**params)
            response.raise_for_status()
            result = response_data_extractor(response)
        except (RetryError, HTTPError, ConnectionError, Timeout, ConnectTimeout) as e:
            raise RequestException(f"HttpClient Request Error: {e}, {e.response.text}")
        except Exception as e:
            raise RequestException(f"HttpClient Error: {e}")
        else:
            return result

    def put(
        self,
        path: str,
        data: dict = None,
        headers: dict = None,
        response_data_extractor: Callable = lambda x: x,
        verify: bool = False,
    ) -> object or Response:
        """
        post 请求
        @param path: 请求路径
        @param data: 请求数据
        @param headers: 请求头
        @param response_data_extractor: 回调函数
        @param verify: 是否校验证书
        @return:
        """
        params = defaultdict()
        params["url"] = path
        params["verify"] = verify
        if headers:
            params["headers"] = headers
        if data:
            params["data"] = data
        try:
            response = self.__requests_retry_session().put(**params)
            response.raise_for_status()
            result = response_data_extractor(response)
        except (RetryError, HTTPError, ConnectionError, Timeout, ConnectTimeout) as e:
            raise RequestException(f"HttpClient Request Error: {e}, {e.response.text}")
        except Exception as e:
            raise RequestException(f"HttpClient Error: {e}")
        else:
            return result



from chatbot.api import ChatBotAPI, Builder
from voluptuous import Required


class AudioChatBot(ChatBotAPI, metaclass=Builder):
    # 语音消息 https://open.feishu.cn/document/uAjLw4CM/ukTMukTMukTM/im-v1/message/create_json#5d353271
    _templates = {
        "basic": {
            Required("msg_type"): "audio",
            Required("content"): {
                Required("file_key"): str
            },
            "receive_id": str,
        }
    }



import copy
import json
from voluptuous import Schema, ALLOW_EXTRA

from chatbot.client import ChatBotClient


class ChatBotAPI:
    def __init__(self, builder):
        self._data = builder.data
        self._client = builder.client
        self._receive_id_type = builder.receive_id_type

    def send(self):
        return self._client.send(self._data, **{"receive_id_type": self._receive_id_type})


class Builder(type):
    receive_id_type = "open_id"
    client, data = None, None

    def __new__(mcs, *args, **kwargs):
        """
        :param args:
        :param kwargs:
        :return:
        """
        cls = type.__new__(mcs, *args, **kwargs)
        return cls

    def set_client(self, client):
        if not isinstance(client, ChatBotClient):
            raise ChatBotException("client must be a concrete implementation of ChatBotClient")
        self.client = client
        return self

    def set_data(self, data):
        if not self._check_message_struct(data):
            raise ChatBotException("message struct error!")
        data["content"] = json.dumps(data["content"])
        self.data = data
        return self

    def set_receive_id_type(self, receive_id_type):
        if receive_id_type not in ["open_id", "user_id", "union_id", "email", "chat_id"]:
            raise ChatBotException(f"Invalid receive_id_type: {receive_id_type}")
        self.receive_id_type = receive_id_type
        return self

    def build(self):
        if not self.client or not self.data:
            raise ChatBotException("build error, client, data, receive_id_type parameter is required")
        return self(self)

    def _check_message_struct(self, message):
        check_success_count = 0
        for _, _schema in self._templates.items():
            try:
                schema = Schema(_schema, True, ALLOW_EXTRA)
                schema(message)
                check_success_count += 1
            except Exception as e:
                print(e)
        return check_success_count > 0


class ChatBotException(Exception):
    pass
